;;; a/d converter driver


;; bit definitions

; ADCSRA
.equ ADEN = 7
.equ ADSC = 6
.equ ADATE = 5
.equ ADIF = 4
.equ ADIE = 3
.equ ADPS2 = 2
.equ ADPS1 = 1
.equ ADPS0 = 0

; ADMUX
.equ REFS1 = 7
.equ REFS0 = 6
.equ ADLAR = 5
.equ MUX4 = 4
.equ MUX3 = 3
.equ MUX2 = 2
.equ MUX1 = 1
.equ MUX0 = 0


;; data buffer

.set adc_data = heap
.set heap = heap + $10


;; adc isr

.set pc_ = pc
.org ADC
  rjmp adc_isr
.org pc_

adc_isr:
  push temp0
  in_ temp0,SREG
  push temp0
  push temp1
  push xl
  push xh

  ; get address for current channel
  ldi xl,low(adc_data)
  ldi xh,high(adc_data)
  in_ temp0,ADMUX
  andi temp0,$07
  lsl temp0
  add xl,temp0
  adc xh,zero

  ; save adc value in ram
  in_ temp0,ADCL
  in_ temp1,ADCH
  st x+,temp0
  st x+,temp1
  
  ; select next channel
  in_ temp0,ADMUX
  mov temp1,temp0
  andi temp0,$f8
  inc temp1
  andi temp1,$07
  or temp0,temp1
  out_ ADMUX,temp0

  ; start next conversion
  sbi_ ADCSRA,ADSC,temp0

  pop xh
  pop xl
  pop temp1
  pop temp0
  out_ SREG,temp0
  pop temp0
  reti


;; adc@
;;
;; ( n1 -- n2 )
;;
;; fetch adc reading

link_
.db "adc@",0,0

adcfetch:
  twostar_
  ldi xl,low(adc_data)
  ldi xh,high(adc_data)
  add xl,tosl
  adc xh,tosh
  cli
  ld tosl,x+
  ld tosh,x+
  sei
  ret


;; adc
;;
;; ( -- )
;;
;; enable adc

link_
.db "adc",0

adcon:

.ifdef DIDR0
  ; disable digital input drivers
  ldi temp0,$ff
  out_ DIDR0,temp0
.endif

  ; start ADC on channel 0, left justify result
  ldi temp0,(1<<ADLAR)
  out_ ADMUX,temp0

  ; enable ADC, ADC interrupt, set prescaler to 128, and start first conversion
  ldi temp0,(1<<ADEN)|(1<<ADSC)|(1<<ADIE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)
  out_ ADCSRA,temp0
  ret


;; /adc
;;
;; ( -- )
;;
;; disable adc

link_
.db "/adc",0,0

notadc:
  out_ ADCSRA,zero

.ifdef DIDR0
  out_ DIDR0,zero
.endif

  ret

